@@ -142,9 +142,21 @@ class Code:
     # Python-callable code for all contained objects in this Code.
     def generate_python(self):
         lines = []
+        type_blocks = set()
         for (_, name) in self.can_contain:
             for instance in getattr(self, name):
-                lines += instance.generate_python() + ['']
+                # TODO: Find a way to generalize this pattern better,
+                #       should not need custom logic like this.
+                args = []
+                if (instance.type in {"SUBROUTINE","FUNCTION","MODULE"}):
+                    args.append(type_blocks)
+                # Get the lines of python code.
+                lines += instance.generate_python(*args) + ['']
+        # Add the type declarations if there are any.
+        for block in sorted(type_blocks):
+            lines.insert(0,"")
+            for line in reversed(block.split("\n")):
+                lines.insert(0, line)
         # Return the full list of lines.
         return lines
 
